@@ -9,6 +9,7 @@ import zipfile
 
 from django.conf import settings
 from django.core.files import File
+from django.core.files.base import ContentFile
 from django.core.files.storage import default_storage
 from django.template import Context, Template
 from django.utils import timezone
@@ -19,6 +20,7 @@ from six import string_types
 
 from web_fragments.fragment import Fragment
 from xblock.core import XBlock
+from xblock.completable import CompletableXBlockMixin
 from xblock.fields import Scope, String, Float, Boolean, Dict, DateTime, Integer
 
 
@@ -31,7 +33,7 @@ logger = logging.getLogger(__name__)
 
 
 @XBlock.wants("settings")
-class ScormXBlock(XBlock):
+class ScormXBlock(XBlock, CompletableXBlockMixin):
     """
     When a user uploads a Scorm package, the zip file is stored in:
 
@@ -45,7 +47,7 @@ class ScormXBlock(XBlock):
         XBLOCK_SETTINGS["ScormXBlock"] = {
             "LOCATION": "alternatevalue",
         }
-    
+
     Note that neither the folder the folder nor the package file are deleted when the
     xblock is removed.
 
@@ -58,7 +60,7 @@ class ScormXBlock(XBlock):
             from django.conf import settings
             from django.core.files.storage import FileSystemStorage
             from openedx.core.djangoapps.site_configuration.models import SiteConfiguration
-            
+
             subfolder = SiteConfiguration.get_value_for_org(
                 xblock.location.org, "SCORM_STORAGE_NAME", "default"
             )
@@ -86,7 +88,12 @@ class ScormXBlock(XBlock):
     lesson_status = String(scope=Scope.user_state, default="not attempted")
     success_status = String(scope=Scope.user_state, default="unknown")
     lesson_score = Float(scope=Scope.user_state, default=0)
-    weight = Float(default=1, scope=Scope.settings)
+    weight = Float(
+        default=1,
+        display_name=_("Weight"),
+        help=_("Weight/Maximum grade"),
+        scope=Scope.settings,
+    )
     has_score = Boolean(
         display_name=_("Scored"),
         help=_(
@@ -113,6 +120,13 @@ class ScormXBlock(XBlock):
         scope=Scope.settings,
     )
 
+    fullscreen_on_launch = Boolean(
+        display_name=_("Fullscreen on launch"),
+        help=_("Display in fullscreen mode on launch"),
+        default=False,
+        scope=Scope.settings,
+    )
+
     has_author_view = True
 
     def render_template(self, template_path, context):
@@ -135,22 +149,40 @@ class ScormXBlock(XBlock):
         return self.student_view(context=context)
 
     def student_view(self, context=None):
-        student_context = self.get_context_student()
+        student_context = {
+            "index_page_url": self.index_page_url,
+            "completion_status": self.get_completion_status(),
+            "grade": self.get_grade(),
+            "scorm_xblock": self,
+        }
         student_context.update(context or {})
         template = self.render_template("static/html/scormxblock.html", student_context)
         frag = Fragment(template)
         frag.add_css(self.resource_string("static/css/scormxblock.css"))
         frag.add_javascript(self.resource_string("static/js/src/scormxblock.js"))
         frag.initialize_js(
-            "ScormXBlock", json_args={"scorm_version": self.scorm_version}
+            "ScormXBlock",
+            json_args={
+                "scorm_version": self.scorm_version,
+                "fullscreen_on_launch": self.fullscreen_on_launch,
+            },
         )
         return frag
 
     def studio_view(self, context=None):
-        # Note that we could make this xblock editable so that we wouldn't need to
-        # create a manual studio view.
-        context = self.get_context_studio()
-        template = self.render_template("static/html/studio.html", context)
+        # Note that we cannot use xblockutils's StudioEditableXBlockMixin because we
+        # need to support package file uploads.
+        studio_context = {
+            "field_display_name": self.fields["display_name"],
+            "field_has_score": self.fields["has_score"],
+            "field_weight": self.fields["weight"],
+            "field_width": self.fields["width"],
+            "field_height": self.fields["height"],
+            "field_fullscreen_on_launch": self.fields["fullscreen_on_launch"],
+            "scorm_xblock": self,
+        }
+        studio_context.update(context or {})
+        template = self.render_template("static/html/studio.html", studio_context)
         frag = Fragment(template)
         frag.add_css(self.resource_string("static/css/scormxblock.css"))
         frag.add_javascript(self.resource_string("static/js/src/studio.js"))
@@ -166,8 +198,10 @@ class ScormXBlock(XBlock):
         self.display_name = request.params["display_name"]
         self.width = request.params["width"]
         self.height = request.params["height"]
-        self.has_score = request.params["has_score"]
-        self.icon_class = "problem" if self.has_score == "True" else "video"
+        self.has_score = request.params["has_score"] == "1"
+        self.weight = request.params["weight"]
+        self.fullscreen_on_launch = request.params["fullscreen_on_launch"] == "1"
+        self.icon_class = "problem" if self.has_score else "video"
 
         response = {"result": "success", "errors": []}
         if not hasattr(request.params["file"], "file"):
@@ -188,8 +222,10 @@ class ScormXBlock(XBlock):
         self.clean_storage()
 
         # Then, extract zip file
+        # At this point we can no longer use the package_file object because some storage backends close the file after
+        # saving. So we need to re-open the file, this time from the storage backend.
         try:
-            self.extract_package(package_file)
+            self.extract_package(self.storage.open(self.package_path))
             self.update_package_fields()
         except ScormError as e:
             response["errors"].append(e.args[0])
@@ -247,7 +283,7 @@ class ScormXBlock(XBlock):
                         )
                         self.storage.save(
                             dest_path,
-                            scorm_zipfile.open(zipinfo.filename),
+                            ContentFile(scorm_zipfile.read(zipinfo.filename)),
                         )
 
     @property
@@ -307,49 +343,61 @@ class ScormXBlock(XBlock):
     def scorm_set_value(self, data, _suffix):
         context = {"result": "success"}
         name = data.get("name")
+        publish_grade = False
+        completion_percent = None
 
         if name in ["cmi.core.lesson_status", "cmi.completion_status"]:
             self.lesson_status = data.get("value")
-            if self.has_score and data.get("value") in [
+            if data.get("value") in [
                 "completed",
                 "failed",
                 "passed",
             ]:
-                self.publish_grade()
-                context.update({"lesson_score": self.lesson_score})
+                completion_percent = 1
+                if self.has_score:
+                    publish_grade = True
         elif name == "cmi.success_status":
             self.success_status = data.get("value")
             if self.has_score:
                 if self.success_status == "unknown":
                     self.lesson_score = 0
-                self.publish_grade()
-                context.update({"lesson_score": self.lesson_score})
+                publish_grade = True
         elif name in ["cmi.core.score.raw", "cmi.score.raw"] and self.has_score:
             self.lesson_score = float(data.get("value", 0)) / 100.0
-            self.publish_grade()
-            context.update({"lesson_score": self.lesson_score})
+            publish_grade = True
         else:
             self.scorm_data[name] = data.get("value", "")
 
+        if completion_percent is not None:
+            self.emit_completion(completion_percent)
+        if publish_grade:
+            self.publish_grade()
+            context.update({"lesson_score": self.lesson_score})
+
         context.update({"completion_status": self.get_completion_status()})
         return context
 
     def publish_grade(self):
+        self.runtime.publish(
+            self,
+            "grade",
+            {"value": self.get_grade(), "max_value": self.weight},
+        )
+
+    def get_grade(self):
+        lesson_score = self.lesson_score
         if self.lesson_status == "failed" or (
             self.scorm_version == "SCORM_2004"
             and self.success_status in ["failed", "unknown"]
         ):
-            self.runtime.publish(self, "grade", {"value": 0, "max_value": self.weight})
-        else:
-            self.runtime.publish(
-                self, "grade", {"value": self.lesson_score, "max_value": self.weight}
-            )
+            lesson_score = 0
+        return lesson_score * self.weight
 
     def set_score(self, score):
         """
         Utility method used to rescore a problem.
         """
-        self.lesson_score = score.raw_earned
+        self.lesson_score = score.raw_earned / self.weight
 
     def max_score(self):
         """
@@ -357,22 +405,6 @@ class ScormXBlock(XBlock):
         """
         return self.weight if self.has_score else None
 
-    def get_context_studio(self):
-        return {
-            "field_display_name": self.fields["display_name"],
-            "field_has_score": self.fields["has_score"],
-            "field_width": self.fields["width"],
-            "field_height": self.fields["height"],
-            "scorm_xblock": self,
-        }
-
-    def get_context_student(self):
-        return {
-            "index_page_url": self.index_page_url,
-            "completion_status": self.get_completion_status(),
-            "scorm_xblock": self,
-        }
-
     def update_package_meta(self, package_file):
         self.package_meta["sha1"] = self.get_sha1(package_file)
         self.package_meta["name"] = package_file.name
@@ -505,14 +537,15 @@ class ScormXBlock(XBlock):
         Return the storage backend used to store the assets of this xblock. This is a cached property.
         """
         if not getattr(self, "_storage", None):
+
             def get_default_storage(_xblock):
                 return default_storage
-        
+
             storage_func = self.xblock_settings.get("STORAGE_FUNC", get_default_storage)
             if isinstance(storage_func, string_types):
                 storage_func = import_string(storage_func)
             self._storage = storage_func(self)
-        
+
         return self._storage
 
     @property
